@@ -1,6 +1,7 @@
 from django.core.exceptions import ValidationError
 from django.db import models
 from django.utils import simplejson
+from django.utils.encoding import smart_unicode
 
 
 class JSONField(models.TextField):
@@ -34,9 +35,13 @@ class JSONField(models.TextField):
         except Exception, e:
             raise ValidationError(str(e))
 
-    def get_db_prep_value(self, value, connection, prepared=False):
+    def get_prep_value(self, value):
         """Convert value to JSON string before save"""
         try:
             return simplejson.dumps(value)
         except Exception, e:
             raise ValidationError(str(e))
+
+    def value_to_string(self, obj):
+        """Return value from object converted to string properly"""
+        return smart_unicode(self.get_prep_value(self._get_val_from_obj(obj)))
